@@ -15,21 +15,9 @@
 // Package daead provides implementations of the DeterministicAEAD primitive.
 //
 // Unlike AEAD, implementations of this interface are not semantically secure, because
-// encrypting the same plaintex always yields the same ciphertext.
+// encrypting the same plaintext always yields the same ciphertext.
 package daead
 
 import (
-	"fmt"
-
-	"github.com/tink-crypto/tink-go/v2/core/registry"
-	"github.com/tink-crypto/tink-go/v2/internal/internalregistry"
+	_ "github.com/tink-crypto/tink-go/v2/daead/aessiv" // Register AES-SIV key manager, key and parameters serialization.
 )
-
-func init() {
-	if err := registry.RegisterKeyManager(new(aesSIVKeyManager)); err != nil {
-		panic(fmt.Sprintf("daead.init() failed: %v", err))
-	}
-	if err := internalregistry.AllowKeyDerivation(aesSIVTypeURL); err != nil {
-		panic(fmt.Sprintf("daead.init() failed: %v", err))
-	}
-}
