@@ -19,6 +19,7 @@ import (
 	"encoding/base64"
 	"fmt"
 	"math/big"
+	"math/bits"
 	"testing"
 
 	"github.com/tink-crypto/tink-go/v2/core/cryptofmt"
@@ -48,17 +49,6 @@ func TestNewParametersInvalidValues(t *testing.T) {
 			},
 			variant: rsassapss.VariantTink,
 		},
-		{
-			name: "large public exponent",
-			parametersValues: rsassapss.ParametersValues{
-				ModulusSizeBits: 2048,
-				SigHashType:     rsassapss.SHA256,
-				MGF1HashType:    rsassapss.SHA256,
-				PublicExponent:  1 << 31,
-				SaltLengthBytes: 1,
-			},
-			variant: rsassapss.VariantTink,
-		},
 		{
 			name: "even public exponent",
 			parametersValues: rsassapss.ParametersValues{
@@ -144,6 +134,23 @@ func TestNewParametersInvalidValues(t *testing.T) {
 			}
 		})
 	}
+
+	// On 32 bit platforms, the public exponent cannot be larger than 1<<31.
+	if bits.UintSize == 64 {
+		expVal := 1 << (bits.UintSize/2 - 1)
+		t.Run("large public exponent", func(t *testing.T) {
+			values := rsassapss.ParametersValues{
+				ModulusSizeBits: 2048,
+				SigHashType:     rsassapss.SHA256,
+				MGF1HashType:    rsassapss.SHA256,
+				PublicExponent:  expVal,
+				SaltLengthBytes: 1,
+			}
+			if _, err := rsassapss.NewParameters(values, rsassapss.VariantTink); err == nil {
+				t.Errorf("rsassapss.NewParameters(%v, %v) = nil, want error", values, rsassapss.VariantTink)
+			}
+		})
+	}
 }
 
 func TestNewParameters(t *testing.T) {
